package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.util.ByteSequence;

public abstract class LocalVariableInstruction extends Instruction implements TypedInstruction, IndexedInstruction {
   /** @deprecated */
   @Deprecated
   protected int n = -1;
   private short cTag = -1;
   private short canonTag = -1;

   LocalVariableInstruction() {
   }

   LocalVariableInstruction(short canonTag, short cTag) {
      this.canonTag = canonTag;
      this.cTag = cTag;
   }

   protected LocalVariableInstruction(short opcode, short cTag, int n) {
      super(opcode, (short)2);
      this.cTag = cTag;
      this.canonTag = opcode;
      this.setIndex(n);
   }

   public void dump(DataOutputStream out) throws IOException {
      if (this.wide()) {
         out.writeByte(196);
      }

      out.writeByte(super.getOpcode());
      if (super.getLength() > 1) {
         if (this.wide()) {
            out.writeShort(this.n);
         } else {
            out.writeByte(this.n);
         }
      }

   }

   public short getCanonicalTag() {
      return this.canonTag;
   }

   public final int getIndex() {
      return this.n;
   }

   public Type getType(ConstantPoolGen cp) {
      switch (this.canonTag) {
         case 21:
         case 54:
            return Type.INT;
         case 22:
         case 55:
            return Type.LONG;
         case 23:
         case 56:
            return Type.FLOAT;
         case 24:
         case 57:
            return Type.DOUBLE;
         case 25:
         case 58:
            return Type.OBJECT;
         case 26:
         case 27:
         case 28:
         case 29:
         case 30:
         case 31:
         case 32:
         case 33:
         case 34:
         case 35:
         case 36:
         case 37:
         case 38:
         case 39:
         case 40:
         case 41:
         case 42:
         case 43:
         case 44:
         case 45:
         case 46:
         case 47:
         case 48:
         case 49:
         case 50:
         case 51:
         case 52:
         case 53:
         default:
            throw new ClassGenException("Unknown case in switch" + this.canonTag);
      }
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      if (wide) {
         this.n = bytes.readUnsignedShort();
         super.setLength(4);
      } else {
         short opcode = super.getOpcode();
         if (opcode >= 21 && opcode <= 25 || opcode >= 54 && opcode <= 58) {
            this.n = bytes.readUnsignedByte();
            super.setLength(2);
         } else {
            if (opcode <= 45) {
               this.n = (opcode - 26) % 4;
            } else {
               this.n = (opcode - 59) % 4;
            }

            super.setLength(1);
         }
      }

   }

   public void setIndex(int n) {
      if (n >= 0 && n <= 65535) {
         this.n = n;
         if (n <= 3) {
            super.setOpcode((short)(this.cTag + n));
            super.setLength(1);
         } else {
            super.setOpcode(this.canonTag);
            if (this.wide()) {
               super.setLength(4);
            } else {
               super.setLength(2);
            }
         }

      } else {
         throw new ClassGenException("Illegal value: " + n);
      }
   }

   final void setIndexOnly(int n) {
      this.n = n;
   }

   public String toString(boolean verbose) {
      short opcode = super.getOpcode();
      return (opcode < 26 || opcode > 45) && (opcode < 59 || opcode > 78) ? super.toString(verbose) + " " + this.n : super.toString(verbose);
   }

   private boolean wide() {
      return this.n > 255;
   }
}
